(**************************************************************************************)
(*  'tils Lib - Librairie de composants divers                                        *)
(*  Copyright (C) 2003  U. Van Den Hekke                                              *)
(*                                                                                    *)
(*  This library is free software; you can redistribute it and/or                     *)
(*  modify it under the terms of the GNU Lesser General Public                        *)
(*  License as published by the Free Software Foundation; either                      *)
(*  version 2.1 of the License, or (at your option) any later version.                *)
(*                                                                                    *)
(*  This library is distributed in the hope that it will be useful,                   *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of                    *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU                 *)
(*  Lesser General Public License for more details.                                   *)
(*                                                                                    *)
(*  You should have received a copy of the GNU Lesser General Public                  *)
(*  License along with this library; if not, write to the Free Software               *)
(*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA         *)
(**************************************************************************************)

unit BaseRegistry;

interface

uses Windows, Variants, Registry;

type
  TRegDataDest  = (rddAuto, rddMachine, rddUser);

  TBaseRegistry = class
  private
    FCurrentKey : HKEY;
    FCurrVer,
    FCompany,
    FAppName,
    FVersion    : string;
    FDebug,
    FFirstStart : boolean;
    FRegistry   : TRegistry;
  protected
    function getApplicationKey: string;
    function getVersionKey: string;
    procedure Default; virtual; abstract;
    function SaveToRegistry: boolean; virtual;
    procedure LoadFromRegistry; virtual;

    function WriteValue(const Name: String; Value: Variant; const DefaultValue: Variant): boolean;
    function  ReadValue(const Name: String; DefaultValue: Variant): Variant;

    property Registry  : TRegistry read FRegistry;
    property CurrentKey: HKEY      read FCurrentKey write FCurrentKey;
  public
    constructor Create(const Dest: TRegDataDest = rddAuto; FileName: string = '');
    destructor Destroy; override;

    property Debug     : boolean read FDebug      write FDebug;
    property FirstStart: boolean read FFirstStart write FFirstStart;
    property Company   : string  read FCompany    write FCompany;
    property AppName   : string  read FAppName    write FAppName;
    property Version   : string  read FVersion    write FVersion;
    property CurrVer   : string  read FCurrVer    write FCurrVer;
  end;

implementation

uses VerManag, WinInfos;

resourcestring
  rsVersion = 'Version';
  rsFirstSt = 'Premier demarrage';
  rsDebug   = 'Debug Version';

const
  dfFirstSt = true;
  dfDebug   = false;

constructor TBaseRegistry.Create(const Dest: TRegDataDest = rddAuto; FileName: string = '');
begin
  FRegistry         := TRegistry.Create;
  if((Dest = rddMachine) or ((Dest = rddAuto) and IsAdminLoggedOn))then
    FCurrentKey     := HKEY_LOCAL_MACHINE
  else
    FCurrentKey     := HKEY_CURRENT_USER;
  FRegistry.RootKey := FCurrentKey;

  if(FileName = '') then FileName := ParamStr(0);
  FAppName          := GetRessourceFile(FileName, viProducteName, '');
  FVersion          := GetRessourceFile(FileName, viProducteVers, '');
  FCompany          := GetRessourceFile(FileName, viCompany, '');
  Default;
end;

destructor TBaseRegistry.Destroy;
begin
  FRegistry.Free;
end;

function TBaseRegistry.getApplicationKey: string;
begin
  result := 'Software\' + FCompany + '\' + FAppName;
end;

function TBaseRegistry.getVersionKey: string;
begin
  result := 'Software\' + FCompany + '\' + FAppName + '\' + FVersion;
end;

function TBaseRegistry.WriteValue(const Name: String; Value: Variant; const DefaultValue: Variant): boolean;
begin
  if(Value = DefaultValue) then begin
    if(FRegistry.ValueExists(Name)) then FRegistry.DeleteValue(Name);
    result := false;
    Exit;
  end;
  result := true;
  case VarType(Value) and varTypeMask of
    varEmpty,
    varNull,
    varUnknown  : ;
    varByte,
    varShortInt,
    varSmallInt,
    varWord,
    varLongWord,
    varInt64,
    varInteger  : FRegistry.WriteInteger(Name, Value);
    varSingle,
    varDouble   : FRegistry.WriteFloat(Name, Value);
    varCurrency : FRegistry.WriteCurrency(Name, Value);
    varDate     : FRegistry.WriteDateTime(Name, Value);
    varBoolean  : FRegistry.WriteBool(Name, Value);
    varString   : FRegistry.WriteString(Name, Value);
  end;
end;

function  TBaseRegistry.ReadValue(const Name: String; DefaultValue: Variant): Variant;
begin
  if not FRegistry.ValueExists(Name) then
  begin
    result := DefaultValue;
    Exit;
  end;
  case VarType(DefaultValue) and varTypeMask of
    varEmpty,
    varNull,
    varUnknown  : ;
    varByte,
    varShortInt,
    varSmallInt,
    varWord,
    varLongWord,
    varInt64,
    varInteger  : result := FRegistry.ReadInteger(Name);
    varSingle,
    varDouble   : result := FRegistry.ReadFloat(Name);
    varCurrency : result := FRegistry.ReadCurrency(Name);
    varDate     : result := FRegistry.ReadDateTime(Name);
    varBoolean  : result := FRegistry.ReadBool(Name);
    varString   : result := FRegistry.ReadString(Name);
  end;
end;

function TBaseRegistry.SaveToRegistry: boolean;
begin
  result := false;
  FFirstStart := false;
  if FRegistry.OpenKey(getApplicationKey, true) then
  try
    WriteValue(rsVersion, FVersion, '');
  finally
    FRegistry.CloseKey;
  end;
  if FRegistry.OpenKey(getVersionKey, true) then
  try
    result := WriteValue(rsFirstSt, FFirstStart, dfFirstSt) or result;
    result := WriteValue(rsDebug, FDebug, dfDebug) or result;
  finally
    FRegistry.CloseKey;
  end;
end;

procedure TBaseRegistry.LoadFromRegistry;
begin
  FFirstStart := dfFirstSt;
  FDebug      := dfDebug;

  if FRegistry.OpenKey(getApplicationKey, false) then
  try
    FCurrVer := ReadValue(rsVersion, '');
    if FCurrVer <> '' then FVersion := FCurrVer;
  finally
    FRegistry.CloseKey;
  end;

  if FRegistry.OpenKey(getVersionKey, false) then
  try
    FFirstStart := ReadValue(rsFirstSt, dfFirstSt);
    FDebug      := ReadValue(rsDebug, dfDebug);
  finally
    FRegistry.CloseKey;
  end;
end;

end.
